@@ -61,7 +61,7 @@ bool wsc_wfa_ext_iter_next(struct wsc_wfa_ext_iter *iter)
 	len = *start;
 	start += 1;
 
-	if (start + len > end)
+	if (len > end - start)
 		return false;
 
 	iter->type = type;
@@ -98,7 +98,7 @@ bool wsc_attr_iter_next(struct wsc_attr_iter *iter)
 	len = l_get_be16(start);
 	start += 2;
 
-	if (start + len > end)
+	if (len > end - start)
 		return false;
 
 	iter->type = type;
@@ -500,7 +500,7 @@ static bool extract_ssid(struct wsc_attr_iter *iter, void *data)
 	unsigned int len;
 
 	len = wsc_attr_iter_get_length(iter);
-	if (len > 32)
+	if (len > SSID_MAX_SIZE)
 		return false;
 
 	ssid->iov_len = len;
@@ -1751,6 +1751,7 @@ static uint8_t *wsc_attr_builder_free(struct wsc_attr_builder *builder,
 
 	if (builder->curlen > 0) {
 		uint8_t *bytes = builder->buf + builder->offset;
+
 		l_put_be16(builder->curlen - 4, bytes + 2);
 		builder->offset += builder->curlen;
 		builder->curlen = 0;
@@ -3073,7 +3074,7 @@ bool wsc_device_type_from_setting_str(const char *value,
 	 * DeviceType=1-0050f204-1
 	 */
 	if (wsc_device_type_from_subcategory_str(out, value))
-		true;
+		return true;
 
 	u = strtoull(value, &endp, 0);
 
